import hashlib, logging, time, os, argparse, sys, struct, operator, subprocess
from solverbase import SolverBase

class ChromeSolver(SolverBase):
    """This class will attempt to solve a captcha by asking chrome speech to text"""

    classname = 'ChromeSolver'
    accuracy_multiplier = .70 
    split_components = [ ]

    decode_url = 'https://www.google.com/speech-api/v1/recognize?lang=en-us&client=chromium'
#    sample_rate = 44100 #Davids summer communications
    #sample_rate = 22050 #Slashdot
    #sample_rate = 11025 #securimage
    def __init__(self, debug = logging.CRITICAL, filename_prefix = "temp/", filename_suffix = ".mp3", skip = 0):
        """ Sets up the chrome solver 
            Parameters: 
                debug = how much information should be printed
                filename_prefix = A filename prefix to add to the beginning of all filenames passed in
                filename_suffix = A filename suffix to add to the end of all the filenames passed in
        """
        self.logger = logging.getLogger(self.classname)
        logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        self.logger.setLevel(debug)

        self.filename_prefix = filename_prefix
        self.filename_suffix = filename_suffix
        self.skip = skip

    def start_solving(self,filename, number_of_words = 1):
        options = [ "sox", self.filename_prefix + filename + self.filename_suffix, self.filename_prefix + filename + ".flac" ]
        if self.skip != 0:
            options.extend(["trim", str(self.skip)])
        options.append("stat")
        sox_process = subprocess.Popen(options, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
        output,error = sox_process.communicate()
        lines = error.split("\n")
        seconds = -1
        samples = -1
        for line in lines:
            parts = line.split(":")
            if parts[0] == "Samples read":
                samples = int(parts[1].replace(" ",""))
            if parts[0] == "Length (seconds)":
                seconds = float(parts[1].replace(" ",""))
        sample_rate = int(round(samples/seconds))

        options = [ "curl", "--data-binary", "@" + self.filename_prefix + filename + ".flac", "--header", 
                        'Content-type: audio/x-flac; rate=' + str(sample_rate), self.decode_url] 
        self.process = subprocess.Popen(options, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
        self.logger.debug("Asking google for the answer")

    def finish_solving(self,filename):
        begin_word = 'utterance":"'
        begin_confidence = 'confidence":'
        output, error = self.process.communicate()
        self.logger.debug("Google answered: %s", output.replace("\n", ""))
        if output.find("Internal Server Error") != -1: #Detect errors from google
            self.logger.info("Google got an internal error")
            return { 0 : { "" : -sys.maxint } }
        elif output.find("NO_MATCH") != -1: #Detect google not being able to find an answer
            self.logger.info("Google couldn't find a match")
            return { 0 : { "" : -sys.maxint } }

        start = output.find(begin_word) + len(begin_word)
        end = output.find('"', start)
        space = output.find(' ', start)
        if space < end and space != -1:
            end = space
        word = output[start:end]
        start = output.find(begin_confidence) + len(begin_confidence) 
        end = output.find("}")
        confidence = output[start:end]
        
        return { 0 : { word : float(confidence) } }


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Asks chrome speech to text to solve mp3 files in the given folder")
    parser.add_argument('dir', type=str, help='The directory to find *.mp3 files in')
    parser.add_argument('-v', required=False, action='store_true', help='Verbose mode.')
    parser.add_argument('-vv', required=False, action='store_true', help='Very Verbose mode.')
    args = parser.parse_args()    

    debug_level = logging.INFO
    if args.v or args.vv:
        debug_level = logging.DEBUG
    files = os.listdir(args.dir)

    chrome_solver = ChromeSolver(debug = debug_level, filename_suffix="", filename_prefix="")
    startTime = time.time()
    for filename in files:
        if filename.endswith(".mp3"):
            chrome_solver.start_solving(args.dir + "/" + filename)
            answer = chrome_solver.finish_solving(args.dir + "/" + filename)
            print filename,"has an answer of",
            for answer_matrix in answer.items():
                word,certainity = max(answer_matrix[1].iteritems(), key=operator.itemgetter(1))
                print word, "with", certainity,"certainity"
            print ""


