import sys, subprocess, Image, time, os, random, logging, shutil

class DownloaderBase():
    """This class will be the parent class that all downloaders inherit from"""

    tempdir = "temp/"
    challenge_field = "recaptcha_challenge_field"
    answer_field = "recaptcha_response_field"
    filetype = "MPEG"
    fileext= ".mp3"
    def __init__(self, proxy, proxy_port, debug = logging.CRITICAL, use_tor = False):
        self.logger = logging.getLogger(self.classname)
        logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        self.logger.setLevel(debug)

        self.tor_process = None
        if use_tor:
            self.start_tor()
            self.proxy = "127.0.0.1"
            self.proxy_port = self.tor_port
        else:
            self.proxy = proxy
            self.proxy_port = proxy_port

        userAgentFile = open("formdata/useragents", "r")
        self.userAgents = userAgentFile.readlines()
        userAgentFile.close()
        self.current_useragent = None
        firstNameFile = open("formdata/fnames", "r")
        self.firstnames = firstNameFile.readlines()
        firstNameFile.close()
        lastNameFile = open("formdata/lnames", "r")
        self.lastnames = lastNameFile.readlines()
        lastNameFile.close()

        emailFile = open("formdata/emailDomains", "r")
        self.emailDomains = emailFile.readlines()
        emailFile.close()

    def __del__(self):
        """Cleans up any remaining processes / files that are left over from downloading captchas"""
        if self.tor_process != None:
            self.tor_process.kill()
            os.remove("torrcs/torrc" + str(self.tor_instance))
            shutil.rmtree("torrcs/torrc" + str(self.tor_instance) + "_dir")

    def submit_captcha(self, challenge, answer):
        """Calls the main respond_to_captcha class with good form data and then determines whether or not it got it correct
            Returns True if it got it correct, False if it got it wrong, and None if it couldn't submit it (banned, etc)"""
        print "Not implemented yet."
        sys.exit()

    def getRandomUsername(self, fname = "", lname = ""):
        symbols = "1234567890__"
        username = ""
        if random.randint(1,10) > 3:
            if fname == "":
                username += self.getRandomFirstName()
            else:
                username += fname
        if username == "" or random.randint(1,10) > 3:
            if fname == "":
                username += self.getRandomLastName()
            else:
                username += lname

        for i in range(0,5):
            if random.randint(1,10) > 3:
                index = random.randint(1,len(username))
                username = username[:index] + symbols[random.randint(0,len(symbols)-1)] + username[index+1:]
        return username 

    def getRandomEmail(self, username = "", fname = "", lname = ""):
        if username == "":
            username = self.getRandomUsername(fname, lname)
        return username + "@" + self.emailDomains[random.randint(0, len(self.emailDomains)-1)][:-1].lower()

    def getRandomFirstName(self):
        return self.firstnames[random.randint(0, len(self.firstnames)-1)][:-1].title()

    def getRandomLastName(self):
        return self.lastnames[random.randint(0, len(self.lastnames)-1)][:-1].title()


    def addRandomUserAgent(self, options, new = False):
        """Adds a random user agent to the curl options.  If new, then it will grab a new random one to use"""
        if new or self.current_useragent == None:
            self.current_useragent = self.userAgents[random.randint(0, len(self.userAgents)-1)][:-1]
        options.extend(["-A", self.current_useragent ])

    def addReferer(self, options):
        if self.referer != "":
            options.extend(["-e", self.referer])

    def addProxy(self, options):
        if self.proxy != "":
            options.extend(["--socks5", self.proxy + ":" + self.proxy_port])

    def get_audio_url_and_challenge(self):
        """Gets the challenge token and corresponding url to download the audio captcha from"""
        self.logger.debug("Downloading challendge")
        url = "http://www.google.com/recaptcha/api/challenge?type=audio&k="
        output = self.get_url(url + self.site_key, True)
        if output.find('</h1>') != -1:
            return "", ""
        challenge = self.find_string(output, "challenge : '", "'")
        return challenge, "http://www.google.com/recaptcha/api/image?c=" + challenge

    def get_image_url_and_challenge(self):
        """Gets the challenge token and corresponding url to download the audio captcha from"""
        self.logger.debug("Downloading challendge")
        url = "http://www.google.com/recaptcha/api/challenge?k="
        output = self.get_url(url + self.site_key, True)
        if output.find('</h1>') != -1:
            return "", ""
        challenge = self.find_string(output, "challenge : '", "'")
        return challenge, "http://www.google.com/recaptcha/api/image?c=" + challenge

    def get_audio(self, url, filename):
        """Get the mp3 file at the corresponding url.  Save the file to the $tempdir/filename.mp3.  Returns whether
            or not the file is an mp3 file"""
        fullfile = self.tempdir + filename + self.fileext
        self.get_file(url, fullfile) 
        return self.is_filetype(fullfile,self.filetype)

    def get_jpeg(self, url, filename):
        """Get the jpeg file at the corresponding url.  Save the file to the $tempdir/filename.mp3.  Returns whether
            or not the file is an mp3 file"""
        fullfile = self.tempdir + filename + ".jpeg"
        self.get_file(url, fullfile) 
        return self.is_filetype(fullfile,"JPEG")

    def get_file(self, url, filename, use_cookies = True):
        options = ["curl", "-s", url, "-o", filename, "-m", "100", "-k" ]
        self.addRandomUserAgent(options)
        self.addReferer(options)
        self.addProxy(options)
        if use_cookies:
            options.extend(["-b", self.tempdir + "cookies" + str(os.getpid())])
        self.logger.debug("Getting %s", filename)
        curl = subprocess.Popen(options, stdin = subprocess.PIPE, stdout = subprocess.PIPE)
        curl.communicate()

    def respond_to_captcha(self, challenge, answer, other_data, use_cookies = True):
        """Submit the captcha and any other data passed in to the submit_url specified when creating the class"""
        self.logger.debug("Submitting captcha")
        data = ""
        if self.challenge_field != "":
            data += self.challenge_field + "=" + challenge + "&"
        data += self.answer_field + "=" + answer
        if other_data != "":
            data += "&" + other_data

        options = ["curl", "-L", "-s", "-d", data, self.submit_url, "-m", "100", "-k"]
        self.addRandomUserAgent(options)
        self.addReferer(options)
        self.addProxy(options)
        if use_cookies:
            options.extend(["-b", self.tempdir + "cookies" + str(os.getpid())])
        curl = subprocess.Popen(options, stdin = subprocess.PIPE, stdout = subprocess.PIPE)
        output,error = curl.communicate()
        return output

    def get_url(self, url, new_useragent = False, get_cookies = True):
        """Download the url and return the contents"""
        options = ["curl", "-s", url, "-m", "100", "-k" ]
        self.addRandomUserAgent(options, new_useragent)
        self.addReferer(options)
        self.addProxy(options)
        if get_cookies:
            options.extend(["-c", self.tempdir + "cookies" + str(os.getpid())])

        curl = subprocess.Popen(options, stdin = subprocess.PIPE, stdout = subprocess.PIPE)
        return curl.communicate()[0]

    def find_string(self, contents, before_string, after_string):
        start = contents.find(before_string)
        start = start + len(before_string)
        if start == -1:
            return ""
        end = contents.find(after_string, start + 1)
        if end == -1:
            return ""
        return contents[start:end]

    def is_filetype(self, filename, filetype):
        """Returns whether or not filename is the specific filetype by running 'file' on it"""
        options = ["file", filename]
        p = subprocess.Popen(options, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
        p.wait()
        output = p.communicate()[0]
        if output.find(filetype) == -1:
            self.logger.debug("Got '" + output.replace("\n","") + "' filetype")
            return False
        return True
        

    def fix_error(self):
        """Take a corrective action to get around being tempbanned if possible"""
        if self.tor_process != None:
            self.logger.debug("Tor is tempbanned, restarting so we get a new exit node")
            self.tor_process.kill()
            self.start_tor()

    def start_tor(self):
        """Start the tor process""" 
        if self.tor_process == None:
            self.tor_instance = 0
            while os.path.exists("torrcs/torrc" + str(self.tor_instance)):
                self.tor_instance += 1
            torrc_original = open("torrcs/torrc", "r")
            contents = torrc_original.read()
            torrc_original.close()

            self.tor_port = str(10000 + self.tor_instance)
            tor_dir = "torrcs/torrc" + str(self.tor_instance) + "_dir"

            contents = contents.replace("REPLACE_WITH_TOR_PORT", self.tor_port)
            contents = contents.replace("REPLACE_WITH_DATA_DIRECTORY", tor_dir)
            torrc_new = open("torrcs/torrc" + str(self.tor_instance), "w")
            torrc_new.write(contents)
            torrc_new.close()

            try:
                os.mkdir(tor_dir)
            except OSError:
                self.logger.info(tor_dir + "already exists.  No need to delete")

        #Starting tor
        self.logger.info("(Re)starting tor with port " + str(self.tor_instance + 10000))
        self.tor_process = subprocess.Popen(["tor", "-f", "torrcs/torrc" + str(self.tor_instance)], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        time.sleep(15) #Give tor time to start
        self.logger.info("Done starting tor")



