%% Setup the parameters you will use for this exercise
%input_layer_size  = 6144;
input_layer_size  = 14336;
hidden_layer_size = 512;
num_labels = 59;
iterations = 500;
bitmask = 32;

printf("Loading training values\n");
cutrate = 0.20
% Load Training Data
% X, each row consists of pixel values for a single input
X = csvread("trainingsetx.csv");
% m, number of samples
m = size(X, 1);

X = X(1:m*cutrate, :);
% y, each row is the index of the word which corresponds to the example in X
printf("Loading answer values\n");
y = csvread("trainingsety.csv");
y = y(1:m*cutrate, :);

% Initialize theta values
% Theta1, takes us from the input layer to the hidden layer
initial_Theta1 = randInitializeWeights(input_layer_size, hidden_layer_size);
% Theta2, takes us from the hidden layer to the output layer
initial_Theta2 = randInitializeWeights(hidden_layer_size, num_labels);
% Unroll parameters
initial_nn_params = [initial_Theta1(:) ; initial_Theta2(:)];

% Train algorith with the training set
options = optimset('MaxIter', iterations);
lambda = 0;
% Create "short hand" for the cost function to be minimized
costFunction = @(p) nnCostFunction(p, ...
                                   input_layer_size, ...
                                   hidden_layer_size, ...
                                   num_labels, X, y, lambda);
% Now, costFunction is a function that takes in only one argument (the
% neural network parameters)
printf("Minimizing cost\n");
[nn_params, cost] = fmincg2(costFunction, initial_nn_params, options, ...
				input_layer_size, hidden_layer_size, ...
				num_labels, bitmask);
% Obtain Theta1 and Theta2 back from nn_params
Theta1 = reshape(nn_params(1:hidden_layer_size * (input_layer_size + 1)), ...
                 hidden_layer_size, (input_layer_size + 1));
Theta2 = reshape(nn_params((1 + (hidden_layer_size * (input_layer_size + 1))):end), ...
                 num_labels, (hidden_layer_size + 1));

save -binary theta_values_32_512_500_20.mat Theta1 Theta2

